use defmt::{info, warn};
use embassy_net::{ConfigV4, Ipv4Address, Ipv4Cidr, Stack};
use embassy_stm32::{gpio::Output, usart::BufferedUart};
use embassy_time::Timer;
use embedded_io::Write;
use embedded_io_async::Read;
use heapless::{String, Vec};

pub async fn net_init(net_key: &mut Output<'static>, usart_modem: &mut BufferedUart<'static>) {
    // Enable the modem
    Timer::after_millis(750).await;
    net_key.set_high();
    Timer::after_millis(750).await;
    net_key.set_low();

    let mut rx_buf = String::<128>::new();
    loop {
        let mut buf = [0u8; 8];
        let result = usart_modem.read(&mut buf).await;
        match result {
            Ok(size) => {
                for i in 0..size {
                    rx_buf.push(buf[i] as char).unwrap();
                }
                if !rx_buf.contains("\r") {
                    continue;
                }
                info!("RX: {}", rx_buf);
                if rx_buf.contains("CTZV") {
                    // usart_modem.write_all(b"AT+QGPS=1\r").unwrap();
                    usart_modem
                        .write_all(b"AT+CGDCONT=1,\"ip\",\"pinternet.interkom.de\"\r")
                        .unwrap();
                    usart_modem.flush().unwrap();
                    Timer::after_millis(500).await;
                    usart_modem.write_all(b"ATD*99#\r").unwrap();
                    usart_modem.flush().unwrap();
                    Timer::after_millis(500).await;
                    break;
                }
                rx_buf.clear();
            }
            Err(_err) => {
                //Ignore eg. framing errors
                defmt::error!("Modem RX Error");
            }
        }
    }
}

#[embassy_executor::task]
pub async fn net_task(
    mut runner: embassy_net::Runner<'static, embassy_net_ppp::Device<'static>>,
) -> ! {
    info!("Starting Ethernet");
    runner.run().await
}

#[embassy_executor::task]
pub async fn ppp_task(
    stack: Stack<'static>,
    mut runner: embassy_net_ppp::Runner<'static>,
    uart: BufferedUart<'static>,
) {
    let cfg = embassy_net_ppp::Config {
        username: "foo".as_bytes(),
        password: "bar".as_bytes(),
    };
    info!("Starting PPP");
    runner
        .run(uart, cfg, |ipv4| {
            let Some(addr) = ipv4.address else {
                warn!("PPP did not provide an IP address.");
                return;
            };
            let mut dns_servers = Vec::new();
            for s in ipv4.dns_servers.iter().flatten() {
                let _ = dns_servers.push(Ipv4Address::from_bytes(&s.0));
            }
            let config = ConfigV4::Static(embassy_net::StaticConfigV4 {
                address: Ipv4Cidr::new(Ipv4Address::from_bytes(&addr.0), 0),
                gateway: None,
                dns_servers,
            });
            stack.set_config_v4(config);
        })
        .await
        .unwrap();
}
