use defmt::info;
use embassy_stm32::i2c::I2c;
use embassy_stm32::mode::Async;
use embassy_time::{Duration, Timer};

const ADDRESS_TMP112: u8 = 0x48;
const ADDRESS_HDC2010: u8 = 0x40;
const ADDRESS_LIGHT: u8 = 0x44;

#[embassy_executor::task]
pub async fn sensors_task(mut i2c: I2c<'static, Async>) {
    info!("Sensors task started");
    Timer::after(Duration::from_secs(1)).await;
    loop {
        info!("TMP112 {}", read_temp_tmp112(&mut i2c).await.unwrap());
        info!("HDC2010 {}", read_temp_hdc2010(&mut i2c).await.unwrap());
        info!("Light {}", read_light(&mut i2c).await.unwrap());
        Timer::after(Duration::from_secs(1)).await;
    }
}

async fn read_light(i2c: &mut I2c<'static, Async>) -> Result<i16, embassy_stm32::i2c::Error> {
    let mut data = [0u8; 2];
    // let _request = i2c.write(ADDRESS_LIGHT, &[0x7E]).await;
    Timer::after(Duration::from_millis(50)).await;
    for i in 1..255 {
        let resp = i2c.write_read(ADDRESS_LIGHT, &[i], &mut data).await;
        match resp {
            Ok(_) => {
                info!("Addr {:02X} Data: {:02X} {:02X}", i, data[0], data[1],);
                return Ok(42);
            }
            Err(e) => match e {
                embassy_stm32::i2c::Error::Nack => {
                    info!("NAK");
                    Timer::after(Duration::from_millis(10)).await;
                }
                _ => {
                    info!("custom err");
                    return Err(e);
                }
            },
        }
    }
    Err(embassy_stm32::i2c::Error::Timeout)
}

async fn read_temp_hdc2010(
    i2c: &mut I2c<'static, Async>,
) -> Result<i16, embassy_stm32::i2c::Error> {
    let mut data = [0u8; 4];
    // let _request = i2c.write(ADDRESS_HUM, &[0x00]).await;
    Timer::after(Duration::from_millis(50)).await;
    for _ in 1..10 {
        let resp = i2c.write_read(ADDRESS_HDC2010, &[0xFC], &mut data).await;
        match resp {
            Ok(_) => {
                info!(
                    "Data: {:02X} {:02X} {:02X} {:02X}",
                    data[0], data[1], data[2], data[3]
                );
                return Ok(42);
            }
            Err(e) => match e {
                embassy_stm32::i2c::Error::Nack => {
                    info!("NAK");
                    Timer::after(Duration::from_millis(10)).await;
                }
                _ => {
                    info!("custom err");
                    return Err(e);
                }
            },
        }
    }
    Err(embassy_stm32::i2c::Error::Timeout)
}

async fn read_temp_tmp112(i2c: &mut I2c<'static, Async>) -> Result<f32, embassy_stm32::i2c::Error> {
    let mut data = [0u8; 2];
    // Timer::after(Duration::from_millis(50)).await;
    for _ in 1..10 {
        let resp = i2c.write_read(ADDRESS_TMP112, &[0], &mut data).await;
        match resp {
            Ok(_) => {
                let temp = ((data[0] as u16) << 4) | (data[1] as u16) >> 4;
                let temp = temp as i16;
                let temp = temp as f32;
                let temp = temp * 0.0625;
                return Ok(temp);
            }
            Err(e) => match e {
                embassy_stm32::i2c::Error::Nack => {
                    info!("NAK");
                    Timer::after(Duration::from_millis(10)).await;
                }
                _ => {
                    info!("custom err");
                    return Err(e);
                }
            },
        }
    }
    Err(embassy_stm32::i2c::Error::Timeout)
}
